﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace FiltersDemo4Blog.Extensions
{
    /// <summary>
    /// Action权限过滤
    /// </summary>
    public class ActionFilters : ActionFilterAttribute
    {
        //参考：www.cnblogs.com/ymnets/p/3452407.html

        public string ActionName { get; set; }   //控制器名
        //public string Area { get; set; }   //区域名
        public string Roles { get; set; }  //可以操作当前Action的角色Id集合

        /// <summary>
        /// 在执行操作方法之前由 MVC 框架调用
        /// </summary>
        public override void OnActionExecuting(ActionExecutingContext filterContext)
        {
            string userName = filterContext.HttpContext.User.Identity.Name;    //当前登录用户的用户名
            Models.User user = Database.SampleData.users.Find(u => u.UserName == userName);   //当前登录用户对象

            if (user != null)
            {
                Models.Role role = Database.SampleData.roles.Find(r => r.Id == user.RoleId);  //当前登录用户的角色

                //获得controller：
                string controllerName = filterContext.RouteData.Values["controller"].ToString().ToLower();
                //string actionName = filterContext.RouteData.Values["action"].ToString().ToLower();
                if (ActionName == null)
                    ActionName = filterContext.RouteData.Values["action"].ToString();

                //查询角色id
                Models.RoleWithControllerAction roleWithControllerAction = Database.SampleData.roleWithControllerAndAction.Find(r => r.ControllerName.ToLower() == controllerName && r.ActionName.ToLower() == ActionName.ToLower());
                if (roleWithControllerAction != null)
                {
                    this.Roles = roleWithControllerAction.RoleIds;     //有权限操作当前控制器和Action的角色id
                }
                if (!string.IsNullOrEmpty(Roles))
                {
                    foreach (string roleid in Roles.Split(','))
                    {
                        if (role.Id.ToString() == roleid)
                            return;   //有权限
                    }
                }
                filterContext.Result = new EmptyResult();   //请求失败输出空结果
                HttpContext.Current.Response.Write("对不起，你没有权限！");   //打出提示文字
            }
            else
            {
                //filterContext.Result = new ViewResult { ViewName = "Error" };
                filterContext.Result = new EmptyResult();
                HttpContext.Current.Response.Write("对不起，请先登录！");
            }
            //base.OnActionExecuting(filterContext);
        }

        /// <summary>
        /// 在执行操作方法后由 MVC 框架调用
        /// </summary>
        public override void OnActionExecuted(ActionExecutedContext filterContext)
        {
            base.OnActionExecuted(filterContext);
        }

        /// <summary>
        /// 在执行操作结果之前由 MVC 框架调用
        /// </summary>
        public override void OnResultExecuting(ResultExecutingContext filterContext)
        {
            base.OnResultExecuting(filterContext);
        }

        /// <summary>
        /// 在执行操作结果后由 MVC 框架调用
        /// </summary>
        public override void OnResultExecuted(ResultExecutedContext filterContext)
        {
            base.OnResultExecuted(filterContext);
        }

    }
}